// ********************************************************************
//
// Filename: ds.c
//
// Version: 1.0
//
// Author: Lachlan O'Dea
//
// Description:
//   A small command-line OS/2 program which displays the amount of
//   disk space available on the specified drive.
//
// Requirements: OS/2 2.0 or greater.
//
// Usage:
//   ds [drive:]
//
// Anyone is free to hack or distribute this code! It's a bit of a
// mess right now.
//
// ********************************************************************


#define INCL_DOSFILEMGR
#include <os2.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

// --- Constants
#define BARWIDTH 70


// --- Function Prototypes
VOID displayerror(APIRET errcode);
INT iround(float f);
VOID DisplayHelp(VOID);

VOID displayerror(APIRET errcode)
{
  PSZ messagestr;

  switch (errcode)
  {
    case 15: messagestr = "Invalid drive was specified"; break;
    case 111: messagestr = "Buffer overflow"; break;
    case 124: messagestr = "Invalid level"; break;
  }
  printf("An error occured when calling DosQueryFSInfo:\n  %s\n", messagestr);
  puts("\n" "--- Program to display disk space ---\n  Usage:\n  ds [drive:]");
}

INT iround(float f)
{
  return f - (INT)f < 0.5 ? (INT)f : (INT)f + 1;
}

void main(SHORT argc, PSZ argv[])
{
  ULONG DriveNumber;  // Drive number
  ULONG FSInfoLevel;  // File system data required
  ULONG FSInfoBuf[5]; // File system info buffer
  ULONG FSInfoBufSize; // File system info buffer size
  APIRET rc; // return code
  UINT i, indicatorwidth;
  ULONG disksize, freespace, used;
  float percentfree;
  ULONG LogicalDriveMap;
  UCHAR achDiskSizeMb[20];
  UCHAR achDiskSizekb[20];
  UCHAR achFreeMb[20];
  UCHAR achFreekb[20];
  UCHAR achUsedMb[20];
  UCHAR achUsedkb[20];

  if (argc <= 1)
    rc = DosQueryCurrentDisk(&DriveNumber, &LogicalDriveMap);
  else
  {
    if (argv[1][0] == '/' || argv[1][0] == '?' || argv[1][0] == '-')
    {
      DisplayHelp();
      exit(EXIT_SUCCESS);
    }
    DriveNumber = toupper(argv[1][0]) - 'A' + 1; //Work out drive number - 1 is A
  }
  
  FSInfoLevel = FSIL_ALLOC; // indicate file system allocation info wanted
  FSInfoBufSize = sizeof(FSInfoBuf);
  rc = DosQueryFSInfo(DriveNumber, FSInfoLevel, (PVOID)FSInfoBuf, FSInfoBufSize);
  if (rc != 0)
  {
    displayerror(rc);
    exit(EXIT_FAILURE);
  }
  else
  {
    disksize = FSInfoBuf[1]*FSInfoBuf[2]*(USHORT)FSInfoBuf[4];
    freespace = FSInfoBuf[1]*FSInfoBuf[3]*(USHORT)FSInfoBuf[4];
    percentfree = (float)FSInfoBuf[3] / (float)FSInfoBuf[2];
    indicatorwidth = (UINT)iround((1.0 - percentfree)*BARWIDTH);
    printf("\n" "**** Disk space on drive %c: ****\n\n", DriveNumber - 1 + 'A');
    printf("   ");
    for (i = 5; i <= BARWIDTH + 4; i++)
      putchar('');
    putchar('');
    printf("\n   ");
    for (i = 5; i < indicatorwidth + 5; i++)
      putchar('');
    for (i = indicatorwidth + 5; i < BARWIDTH + 5; i++)
      putchar(' ');
    putchar('');
    printf("\n   ");
    for (i = 5; i <= BARWIDTH + 4; i++)
      putchar('');
    putchar('');
    printf("\n\n" "%6.2f%% free\n\n", percentfree*100);
    used = disksize - freespace;
    
    sprintf(achDiskSizeMb, "%.2f", (float)disksize/1024.0/1024.0);
    sprintf(achDiskSizekb, "%lu", disksize/1024);
    sprintf(achUsedMb, "%.2f", (float)used/1024.0/1024.0);
    sprintf(achUsedkb, "%lu", used/1024);
    sprintf(achFreeMb, "%.2f", (float)freespace/1024.0/1024.0);
    sprintf(achFreekb, "%lu", freespace/1024);
    printf("Size of disk: %8s Mb %8s kb\n"
           "Space used:   %8s Mb %8s kb\n"
           "-------------------------------------\n"
           "Space free:   %8s Mb %8s kb\n", achDiskSizeMb, achDiskSizekb,
           achUsedMb, achUsedkb, achFreeMb, achFreekb);
    exit(EXIT_SUCCESS);
  } // else
} // main

VOID DisplayHelp(VOID)
{
  puts("\n" "DS - displays available disk space\n"
       "  Usage: ds [drive:]\n"
       "         ds /? for this help");
}
